"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("./common");
const os = require("os");
const tmp = require("tmp");
const fs = require("fs");
const ChildProcess = require("child_process");
const events_1 = require("events");
class OpenOCDServerController extends events_1.EventEmitter {
    constructor() {
        super();
        this.portsNeeded = ['gdbPort'];
        this.name = 'OpenOCD';
        this.swoPath = tmp.tmpNameSync();
    }
    setPorts(ports) {
        this.ports = ports;
    }
    setArguments(args) {
        this.args = args;
    }
    customRequest(command, response, args) {
        return false;
    }
    initCommands() {
        const gdbport = this.ports['gdbPort'];
        return [
            `target-select extended-remote localhost:${gdbport}`
        ];
    }
    launchCommands() {
        const commands = [
            'interpreter-exec console "monitor reset halt; flash erase_sector 0 0 1"',
            'target-download',
            'interpreter-exec console "monitor reset halt"',
            'enable-pretty-printing'
        ];
        if (this.args.swoConfig.enabled) {
            const swocommands = this.SWOConfigurationCommands();
            commands.push(...swocommands);
        }
        return commands;
    }
    attachCommands() {
        const commands = [
            'interpreter-exec console "monitor halt"',
            'enable-pretty-printing'
        ];
        if (this.args.swoConfig.enabled) {
            const swocommands = this.SWOConfigurationCommands();
            commands.push(...swocommands);
        }
        return commands;
    }
    restartCommands() {
        const commands = [
            'interpreter-exec console "monitor reset halt"'
        ];
        if (this.args.swoConfig.enabled) {
            const swocommands = this.SWOConfigurationCommands();
            commands.push(...swocommands);
        }
        return commands;
    }
    SWOConfigurationCommands() {
        const portMask = '0x' + common_1.calculatePortMask(this.args.swoConfig.decoders).toString(16);
        const swoFrequency = this.args.swoConfig.swoFrequency;
        const cpuFrequency = this.args.swoConfig.cpuFrequency;
        const ratio = Math.floor(cpuFrequency / swoFrequency) - 1;
        const commands = [
            'EnableITMAccess',
            `BaseSWOSetup ${ratio}`,
            'SetITMId 1',
            'ITMDWTTransferEnable',
            'DisableITMPorts 0xFFFFFFFF',
            `EnableITMPorts ${portMask}`,
            'EnableDWTSync',
            'ITMSyncEnable',
            'ITMGlobalEnable'
        ];
        commands.push(this.args.swoConfig.profile ? 'EnablePCSample' : 'DisablePCSample');
        return commands.map((c) => `interpreter-exec console "${c}"`);
    }
    serverExecutable() {
        if (this.args.serverpath) {
            return this.args.serverpath;
        }
        else {
            return os.platform() === 'win32' ? 'openocd.exe' : 'openocd';
        }
    }
    serverArguments() {
        const gdbport = this.ports['gdbPort'];
        const serverargs = [];
        serverargs.push('-c', `gdb_port ${gdbport}`);
        this.args.searchDir.forEach((cs, idx) => {
            serverargs.push('-s', cs);
        });
        if (this.args.searchDir.length === 0) {
            serverargs.push('-s', this.args.cwd);
        }
        this.args.configFiles.forEach((cf, idx) => {
            serverargs.push('-f', cf);
        });
        if (this.args.rtos) {
            const tmpCfgPath = tmp.tmpNameSync();
            fs.writeFileSync(tmpCfgPath, `$_TARGETNAME configure -rtos ${this.args.rtos}\n`, 'utf8');
            serverargs.push('-f', tmpCfgPath);
        }
        const commands = [];
        if (this.args.swoConfig.enabled) {
            if (os.platform() == 'win32') {
                this.swoPath = this.swoPath.replace(/\\/g, '/');
            }
            // tslint:disable-next-line:max-line-length
            commands.push(`tpiu config internal ${this.swoPath} uart off ${this.args.swoConfig.cpuFrequency} ${this.args.swoConfig.swoFrequency}`);
        }
        // Append additional commands
        if (this.args.openOCDLaunchCommands) {
            this.args.openOCDLaunchCommands.forEach((cmd) => {
                commands.push(cmd);
            });
        }
        if (commands.length > 0) {
            serverargs.push('-c', commands.join('; '));
        }
        return serverargs;
    }
    initMatch() {
        /*
        // Following will work with or without the -d flag to openocd or using the tcl
        // command `debug_level 3`; and we are looking specifically for gdb port(s) opening up
        // When debug is enabled, you get too many matches looking for the cpu. This message
        // has been there atleast since 2016-12-19
        */
        return /Info\s:[^\n]*Listening on port \d+ for gdb connection/i;
    }
    serverLaunchStarted() {
        if (this.args.swoConfig.enabled && this.args.swoConfig.source === 'probe' && os.platform() !== 'win32') {
            const mkfifoReturn = ChildProcess.spawnSync('mkfifo', [this.swoPath]);
            this.emit('event', new common_1.SWOConfigureEvent({ type: 'fifo', path: this.swoPath }));
        }
    }
    serverLaunchCompleted() {
        if (this.args.swoConfig.enabled) {
            if (this.args.swoConfig.source === 'probe' && os.platform() === 'win32') {
                this.emit('event', new common_1.SWOConfigureEvent({ type: 'file', path: this.swoPath }));
            }
            else if (this.args.swoConfig.source !== 'probe') {
                this.emit('event', new common_1.SWOConfigureEvent({
                    type: 'serial',
                    device: this.args.swoConfig.source,
                    baudRate: this.args.swoConfig.swoFrequency
                }));
            }
        }
    }
    debuggerLaunchStarted() { }
    debuggerLaunchCompleted() { }
}
exports.OpenOCDServerController = OpenOCDServerController;
//# sourceMappingURL=openocd.js.map
